@@ -50,6 +50,7 @@ use constant {
     ALL_PKG => CTRL_INFO_PKG | CTRL_INDEX_PKG | CTRL_PKG_DEB | CTRL_FILE_STATUS,
     ALL_SRC => CTRL_INFO_SRC | CTRL_INDEX_SRC | CTRL_PKG_SRC,
     ALL_CHANGES => CTRL_FILE_CHANGES | CTRL_CHANGELOG,
+    ALL_COPYRIGHT => CTRL_COPYRIGHT_HEADER | CTRL_COPYRIGHT_FILES | CTRL_COPYRIGHT_LICENSE,
 };
 
 use constant {
@@ -68,6 +69,10 @@ our %FIELDS = (
         allowed => (ALL_PKG | ALL_SRC | CTRL_FILE_CHANGES) & (~CTRL_INFO_SRC),
         separator => FIELD_SEP_SPACE,
     },
+    'Architectures' => {
+        allowed => CTRL_REPO_RELEASE,
+        separator => FIELD_SEP_SPACE,
+    },
     'Binary' => {
         allowed => CTRL_PKG_SRC | CTRL_FILE_CHANGES,
         # XXX: This field values are separated either by space or comma
@@ -122,6 +127,9 @@ our %FIELDS = (
         dependency => 'normal',
         dep_order => 3,
     },
+    'Build-Essential' => {
+        allowed => ALL_PKG,
+    },
     'Build-Profiles' => {
         allowed => CTRL_INFO_PKG,
         separator => FIELD_SEP_SPACE,
@@ -139,13 +147,30 @@ our %FIELDS = (
     'Changed-By' => {
         allowed => CTRL_FILE_CHANGES,
     },
+    'Changelogs' => {
+        allowed => CTRL_REPO_RELEASE,
+    },
     'Changes' => {
         allowed => ALL_CHANGES,
     },
+    'Classes' => {
+        allowed => CTRL_TESTS,
+        separator => FIELD_SEP_COMMA,
+    },
     'Closes' => {
         allowed => ALL_CHANGES,
         separator => FIELD_SEP_SPACE,
     },
+    'Codename' => {
+        allowed => CTRL_REPO_RELEASE,
+    },
+    'Comment' => {
+        allowed => ALL_COPYRIGHT,
+    },
+    'Components' => {
+        allowed => CTRL_REPO_RELEASE,
+        separator => FIELD_SEP_SPACE,
+    },
     'Conffiles' => {
         allowed => CTRL_FILE_STATUS,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
@@ -159,17 +184,23 @@ our %FIELDS = (
         dependency => 'union',
         dep_order => 6,
     },
+    'Copyright' => {
+        allowed => CTRL_COPYRIGHT_HEADER | CTRL_COPYRIGHT_FILES,
+    },
     'Date' => {
-        allowed => ALL_CHANGES,
+        allowed => ALL_CHANGES | CTRL_REPO_RELEASE,
     },
     'Depends' => {
-        allowed => ALL_PKG,
+        allowed => ALL_PKG | CTRL_TESTS,
         separator => FIELD_SEP_COMMA,
         dependency => 'normal',
         dep_order => 2,
     },
     'Description' => {
-        allowed => ALL_PKG | CTRL_FILE_CHANGES,
+        allowed => ALL_PKG | CTRL_FILE_CHANGES | CTRL_REPO_RELEASE,
+    },
+    'Disclaimer' => {
+        allowed => CTRL_COPYRIGHT_HEADER,
     },
     'Directory' => {
         allowed => CTRL_INDEX_SRC,
@@ -186,16 +217,20 @@ our %FIELDS = (
     'Essential' => {
         allowed => ALL_PKG,
     },
+    'Features' => {
+        allowed => CTRL_TESTS,
+        separator => FIELD_SEP_SPACE,
+    },
     'Filename' => {
         allowed => CTRL_INDEX_PKG,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'Files' => {
-        allowed => CTRL_PKG_SRC | CTRL_FILE_CHANGES,
+        allowed => CTRL_PKG_SRC | CTRL_FILE_CHANGES | CTRL_COPYRIGHT_FILES,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'Format' => {
-        allowed => CTRL_PKG_SRC | CTRL_FILE_CHANGES,
+        allowed => CTRL_PKG_SRC | CTRL_FILE_CHANGES | CTRL_COPYRIGHT_HEADER,
     },
     'Homepage' => {
         allowed => ALL_SRC | ALL_PKG,
@@ -209,11 +244,17 @@ our %FIELDS = (
     'Kernel-Version' => {
         allowed => ALL_PKG,
     },
+    'Label' => {
+        allowed => CTRL_REPO_RELEASE,
+    },
+    'License' => {
+        allowed => ALL_COPYRIGHT,
+    },
     'Origin' => {
-        allowed => (ALL_PKG | ALL_SRC) & (~CTRL_INFO_PKG),
+        allowed => (ALL_PKG | ALL_SRC | CTRL_REPO_RELEASE) & (~CTRL_INFO_PKG),
     },
     'Maintainer' => {
-        allowed => CTRL_PKG_DEB | ALL_SRC | ALL_CHANGES,
+        allowed => CTRL_PKG_DEB| CTRL_FILE_STATUS | ALL_SRC  | ALL_CHANGES,
     },
     'Multi-Arch' => {
         allowed => ALL_PKG,
@@ -258,6 +299,10 @@ our %FIELDS = (
         dependency => 'union',
         dep_order => 8,
     },
+    'Restrictions' => {
+        allowed => CTRL_TESTS,
+        separator => FIELD_SEP_SPACE,
+    },
     'Section' => {
         allowed => CTRL_INFO_SRC | CTRL_INDEX_SRC | ALL_PKG,
     },
@@ -266,7 +311,7 @@ our %FIELDS = (
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'Source' => {
-        allowed => (ALL_PKG | ALL_SRC | ALL_CHANGES) &
+        allowed => (ALL_PKG | ALL_SRC | ALL_CHANGES | CTRL_COPYRIGHT_HEADER) &
                    (~(CTRL_INDEX_SRC | CTRL_INFO_PKG)),
     },
     'Standards-Version' => {
@@ -279,6 +324,9 @@ our %FIELDS = (
     'Subarchitecture' => {
         allowed => ALL_PKG,
     },
+    'Suite' => {
+        allowed => CTRL_REPO_RELEASE,
+    },
     'Suggests' => {
         allowed => ALL_PKG,
         separator => FIELD_SEP_COMMA,
@@ -292,10 +340,27 @@ our %FIELDS = (
     'Task' => {
         allowed => ALL_PKG,
     },
+    'Test-Command' => {
+        allowed => CTRL_TESTS,
+    },
+    'Tests' => {
+        allowed => CTRL_TESTS,
+        separator => FIELD_SEP_SPACE,
+    },
+    'Tests-Directory' => {
+        allowed => CTRL_TESTS,
+    },
     'Testsuite' => {
         allowed => ALL_SRC,
         separator => FIELD_SEP_COMMA,
     },
+    'Testsuite-Triggers' => {
+        allowed => ALL_SRC,
+        separator => FIELD_SEP_COMMA,
+    },
+    'Timestamp' => {
+        allowed => CTRL_CHANGELOG,
+    },
     'Triggers-Awaited' => {
         allowed => CTRL_FILE_STATUS,
         separator => FIELD_SEP_SPACE,
@@ -308,9 +373,18 @@ our %FIELDS = (
         allowed => ALL_SRC,
         separator => FIELD_SEP_COMMA,
     },
+    'Upstream-Name' => {
+        allowed => CTRL_COPYRIGHT_HEADER,
+    },
+    'Upstream-Contact' => {
+        allowed => CTRL_COPYRIGHT_HEADER,
+    },
     'Urgency' => {
         allowed => ALL_CHANGES,
     },
+    'Valid-Until' => {
+        allowed => CTRL_REPO_RELEASE,
+    },
     'Vcs-Browser' => {
         allowed => ALL_SRC,
     },
@@ -354,7 +428,7 @@ my @checksum_fields = map { &field_capitalize("Checksums-$_") } checksums_get_li
 my @sum_fields = map { $_ eq 'md5' ? 'MD5sum' : &field_capitalize($_) }
                  checksums_get_list();
 &field_register($_, CTRL_PKG_SRC | CTRL_FILE_CHANGES) foreach @checksum_fields;
-&field_register($_, CTRL_INDEX_PKG,
+&field_register($_, CTRL_INDEX_PKG | CTRL_REPO_RELEASE,
                 separator => FIELD_SEP_LINE | FIELD_SEP_SPACE) foreach @sum_fields;
 
 our %FIELD_ORDER = (
@@ -369,8 +443,8 @@ our %FIELD_ORDER = (
         qw(Format Source Binary Architecture Version Origin Maintainer
         Uploaders Homepage Standards-Version Vcs-Browser
         Vcs-Arch Vcs-Bzr Vcs-Cvs Vcs-Darcs Vcs-Git Vcs-Hg Vcs-Mtn
-        Vcs-Svn Testsuite), &field_list_src_dep(), qw(Package-List),
-        @checksum_fields, qw(Files)
+        Vcs-Svn Testsuite Testsuite-Triggers), &field_list_src_dep(),
+        qw(Package-List), @checksum_fields, qw(Files)
     ],
     CTRL_FILE_CHANGES() => [
         qw(Format Date Source Binary Binary-Only Built-For-Profiles Architecture
@@ -380,7 +454,7 @@ our %FIELD_ORDER = (
     ],
     CTRL_CHANGELOG() => [
         qw(Source Binary-Only Version Distribution Urgency Maintainer
-        Date Closes Changes)
+        Timestamp Date Closes Changes)
     ],
     CTRL_FILE_STATUS() => [ # Same as fieldinfos in lib/dpkg/parse.c
         qw(Package Essential Status Priority Section Installed-Size Origin
@@ -389,6 +463,20 @@ our %FIELD_ORDER = (
         Conflicts Enhances Conffiles Description Triggers-Pending
         Triggers-Awaited)
     ],
+    CTRL_REPO_RELEASE() => [
+        qw(Origin Label Suite Codename Changelogs Date Valid-Until
+        Architectures Components Description), @sum_fields
+    ],
+    CTRL_COPYRIGHT_HEADER() => [
+        qw(Format Upstream-Name Upstream-Contact Source Disclaimer Comment
+        License Copyright)
+    ],
+    CTRL_COPYRIGHT_FILES() => [
+        qw(Files Copyright License Comment)
+    ],
+    CTRL_COPYRIGHT_LICENSE() => [
+        qw(License Comment)
+    ],
 );
 # Order for CTRL_INDEX_PKG is derived from CTRL_PKG_DEB
 $FIELD_ORDER{CTRL_INDEX_PKG()} = [ @{$FIELD_ORDER{CTRL_PKG_DEB()}} ];
@@ -416,7 +504,7 @@ CTRL_* constants exported by Dpkg::Control.
 
 =over 4
 
-=item my $f = field_capitalize($field_name)
+=item $f = field_capitalize($field_name)
 
 Returns the field name properly capitalized. All characters are lowercase,
 except the first of each word (words are separated by a hyphen in field names).
@@ -478,7 +566,7 @@ $from Dpkg::Control object to the $to Dpkg::Control object.
 Official fields are copied only if the field is allowed in both types of
 objects. Custom fields are treated in a specific manner. When the target
 is not among CTRL_PKG_SRC, CTRL_PKG_DEB or CTRL_FILE_CHANGES, then they
-are alway copied as is (the X- prefix is kept). Otherwise they are not
+are always copied as is (the X- prefix is kept). Otherwise they are not
 copied except if the target object matches the target destination encoded
 in the field name. The initial X denoting custom fields can be followed by
 one or more letters among "S" (Source: corresponds to CTRL_PKG_SRC), "B"
@@ -679,10 +767,6 @@ sub field_insert_before($$@) {
 
 Mark the module as public.
 
-=head1 AUTHOR
-
-Raphaël Hertzog <hertzog@debian.org>.
-
 =cut
 
 1;
