@@ -65,7 +65,7 @@ spaces are stripped only on the first line of each field.
 
 =over 4
 
-=item my $c = Dpkg::Control::Hash->new(%opts)
+=item $c = Dpkg::Control::Hash->new(%opts)
 
 Creates a new object with the indicated options. Supported options
 are:
@@ -149,7 +149,7 @@ sub set_options {
     $$self->{$_} = $opts{$_} foreach keys %opts;
 }
 
-=item my $value = $c->get_option($option)
+=item $value = $c->get_option($option)
 
 Returns the value of the corresponding option.
 
@@ -220,10 +220,10 @@ sub parse {
 	    unless (defined($cf)) {
 		$self->parse_error($desc, g_('continued value line not in field'));
             }
+	    $line =~ s/\s*$//;
 	    if ($line =~ /^\.+$/) {
 		$line = substr $line, 1;
 	    }
-	    $line =~ s/\s*$//;
 	    $self->{$cf} .= "\n$line";
 	} elsif (m/^-----BEGIN PGP SIGNED MESSAGE-----[\r\t ]*$/) {
 	    $expect_pgp_sig = 1;
@@ -309,7 +309,7 @@ sub get_custom_field {
 Write the string representation of the control information to a
 file.
 
-=item my $str = $c->output()
+=item $str = $c->output()
 
 =item "$c"
 
@@ -444,7 +444,6 @@ package Dpkg::Control::HashCore::Tie;
 use strict;
 use warnings;
 
-use Dpkg::Checksums;
 use Dpkg::Control::FieldsCore;
 
 use Carp;
@@ -550,10 +549,6 @@ New method: $c->parse_error().
 
 Mark the module as public.
 
-=head1 AUTHOR
-
-Raphaël Hertzog <hertzog@debian.org>.
-
 =cut
 
 1;
