@@ -1,7 +1,7 @@
 /*
  * update-alternatives
  *
- * Copyright © 1995 Ian Jackson <ian@davenant.greenend.org.uk>
+ * Copyright © 1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2000-2002 Wichert Akkerman <wakkerma@debian.org>
  * Copyright © 2006-2015 Guillem Jover <guillem@debian.org>
  * Copyright © 2008 Pierre Habouzit <madcoder@debian.org>
@@ -63,8 +63,6 @@ static int opt_skip_auto = 0;
 static int opt_verbose = 0;
 static int opt_force = 0;
 
-#define ALT_TMP_EXT ".dpkg-tmp"
-
 /*
  * Functions.
  */
@@ -316,29 +314,28 @@ xreadlink(const char *linkname)
 	return buf;
 }
 
-static int DPKG_ATTR_VPRINTF(2)
-xvasprintf(char **strp, const char *fmt, va_list args)
+static char * DPKG_ATTR_VPRINTF(1)
+xvasprintf(const char *fmt, va_list args)
 {
-	int ret;
+	char *str;
 
-	ret = vasprintf(strp, fmt, args);
-	if (ret < 0)
+	if (vasprintf(&str, fmt, args) < 0)
 		error(_("failed to allocate memory"));
 
-	return ret;
+	return str;
 }
 
-static int DPKG_ATTR_PRINTF(2)
-xasprintf(char **strp, const char *fmt, ...)
+static char * DPKG_ATTR_PRINTF(1)
+xasprintf(const char *fmt, ...)
 {
 	va_list args;
-	int ret;
+	char *str;
 
 	va_start(args, fmt);
-	ret = xvasprintf(strp, fmt, args);
+	str = xvasprintf(fmt, args);
 	va_end(args);
 
-	return ret;
+	return str;
 }
 
 static void
@@ -353,18 +350,17 @@ set_action(const char *new_action)
 static const char *
 admindir_init(void)
 {
-	const char *basedir, *dpkg_basedir;
-	char *admindir;
+	const char *basedir, *basedir_env;
 
-	dpkg_basedir = getenv("DPKG_ADMINDIR");
-	if (dpkg_basedir)
-		basedir = dpkg_basedir;
+	/* Try to get the admindir from an environment variable, usually set
+	 * by the system package manager. */
+	basedir_env = getenv(ADMINDIR_ENVVAR);
+	if (basedir_env)
+		basedir = basedir_env;
 	else
 		basedir = ADMINDIR;
 
-	xasprintf(&admindir, "%s/%s", basedir, "alternatives");
-
-	return admindir;
+	return xasprintf("%s/%s", basedir, "alternatives");
 }
 
 static FILE *fh_log = NULL;
@@ -464,7 +460,7 @@ checked_rm_args(const char *fmt, ...)
 	char *path;
 
 	va_start(args, fmt);
-	xvasprintf(&path, fmt, args);
+	path = xvasprintf(fmt, args);
 	va_end(args);
 
 	checked_rm(path);
@@ -1105,7 +1101,7 @@ altdb_parse_error(struct altdb_context *ctx, const char *format, ...)
 	va_list args;
 
 	va_start(args, format);
-	xvasprintf(&msg, format, args);
+	msg = xvasprintf(format, args);
 	va_end(args);
 
 	error(_("%s corrupt: %s"), ctx->filename, msg);
@@ -1139,8 +1135,8 @@ alternative_parse_slave(struct alternative *a, struct altdb_context *ctx)
 		free(name);
 		return false;
 	}
-	if (alternative_has_slave(a, name)) {
-		sl = alternative_get_slave(a, name);
+	sl = alternative_get_slave(a, name);
+	if (sl) {
 		free(name);
 		ctx->bad_format(ctx, _("duplicate slave name %s"), sl->name);
 	}
@@ -1236,7 +1232,7 @@ alternative_load(struct alternative *a, enum altdb_flags flags)
 {
 	struct altdb_context ctx;
 	struct stat st;
-	char *fn, *status;
+	char *status;
 
 	/* Initialize parse context */
 	if (setjmp(ctx.on_error)) {
@@ -1252,8 +1248,7 @@ alternative_load(struct alternative *a, enum altdb_flags flags)
 		ctx.bad_format = altdb_parse_stop;
 	else
 		ctx.bad_format = altdb_parse_error;
-	xasprintf(&fn, "%s/%s", admdir, a->master_name);
-	ctx.filename = fn;
+	ctx.filename = xasprintf("%s/%s", admdir, a->master_name);
 
 	/* Open the alternative file. */
 	ctx.fh = fopen(ctx.filename, "r");
@@ -1342,8 +1337,8 @@ alternative_save(struct alternative *a)
 	alternative_sort_choices(a);
 
 	/* Write admin file. */
-	xasprintf(&file, "%s/%s", admdir, a->master_name);
-	xasprintf(&filenew, "%s" ALT_TMP_EXT, file);
+	file = xasprintf("%s/%s", admdir, a->master_name);
+	filenew = xasprintf("%s" ALT_TMP_EXT, file);
 
 	ctx.filename = filenew;
 	ctx.fh = fopen(ctx.filename, "w");
@@ -1363,7 +1358,7 @@ alternative_save(struct alternative *a)
 
 		altdb_print_line(&ctx, fs->master_file);
 
-		xasprintf(&prio, "%d", fs->priority);
+		prio = xasprintf("%d", fs->priority);
 		altdb_print_line(&ctx, prio);
 		free(prio);
 
@@ -1411,7 +1406,7 @@ alternative_get_current(struct alternative *a)
 	if (a->known_current)
 		return a->current;
 
-	xasprintf(&curlink, "%s/%s", altdir, a->master_name);
+	curlink = xasprintf("%s/%s", altdir, a->master_name);
 	if (lstat(curlink, &st)) {
 		if (errno == ENOENT) {
 			free(curlink);
@@ -1749,8 +1744,8 @@ alternative_prepare_install_single(struct alternative *a, const char *name,
 	char *fntmp, *fn;
 
 	/* Create link in /etc/alternatives. */
-	xasprintf(&fntmp, "%s/%s" ALT_TMP_EXT, altdir, name);
-	xasprintf(&fn, "%s/%s", altdir, name);
+	fntmp = xasprintf("%s/%s" ALT_TMP_EXT, altdir, name);
+	fn = xasprintf("%s/%s", altdir, name);
 	checked_rm(fntmp);
 	checked_symlink(file, fntmp);
 	alternative_add_commit_op(a, OPCODE_MV, fntmp, fn);
@@ -1758,7 +1753,7 @@ alternative_prepare_install_single(struct alternative *a, const char *name,
 
 	if (alternative_path_needs_update(linkname, fn)) {
 		/* Create alternative link. */
-		xasprintf(&fntmp, "%s" ALT_TMP_EXT, linkname);
+		fntmp = xasprintf("%s" ALT_TMP_EXT, linkname);
 		checked_rm(fntmp);
 		checked_symlink(fn, fntmp);
 		alternative_add_commit_op(a, OPCODE_MV, fntmp, linkname);
@@ -1799,7 +1794,7 @@ alternative_prepare_install(struct alternative *a, const char *choice)
 			        a->master_name);
 
 		/* Drop unused slave. */
-		xasprintf(&fn, "%s/%s", altdir, sl->name);
+		fn = xasprintf("%s/%s", altdir, sl->name);
 		if (alternative_path_can_remove(sl->link))
 			alternative_add_commit_op(a, OPCODE_RM, sl->link, NULL);
 		else
@@ -1876,7 +1871,7 @@ alternative_has_broken_slave(struct slave_link *sl, struct fileset *fs)
 		sl_altlnk = areadlink(sl->link);
 		if (!sl_altlnk)
 			return true;
-		xasprintf(&wanted, "%s/%s", altdir, sl->name);
+		wanted = xasprintf("%s/%s", altdir, sl->name);
 		if (strcmp(sl_altlnk, wanted) != 0) {
 			free(wanted);
 			free(sl_altlnk);
@@ -1899,7 +1894,7 @@ alternative_has_broken_slave(struct slave_link *sl, struct fileset *fs)
 		/* Slave link must not exist. */
 		if (alternative_path_classify(sl->link) != ALT_PATH_MISSING)
 			return true;
-		xasprintf(&sl_altlnk, "%s/%s", altdir, sl->name);
+		sl_altlnk = xasprintf("%s/%s", altdir, sl->name);
 		if (alternative_path_classify(sl_altlnk) != ALT_PATH_MISSING) {
 			free(sl_altlnk);
 			return true;
@@ -1923,7 +1918,7 @@ alternative_needs_update(struct alternative *a)
 	altlnk = areadlink(a->master_link);
 	if (!altlnk)
 		return ALT_UPDATE_LINK_BROKEN;
-	xasprintf(&wanted, "%s/%s", altdir, a->master_name);
+	wanted = xasprintf("%s/%s", altdir, a->master_name);
 	if (strcmp(altlnk, wanted) != 0) {
 		free(wanted);
 		free(altlnk);
@@ -2154,11 +2149,18 @@ static void
 alternative_evolve_slave(struct alternative *a, const char *cur_choice,
                          struct slave_link *sl, struct fileset *fs)
 {
+	struct slave_link *sl_old;
 	struct stat st;
 	char *new_file = NULL;
 	const char *old, *new;
 
-	old = alternative_get_slave(a, sl->name)->link;
+	sl_old = alternative_get_slave(a, sl->name);
+	if (sl_old == NULL) {
+		sl->updated = true;
+		return;
+	}
+
+	old = sl_old->link;
 	new = sl->link;
 
 	if (cur_choice && strcmp(cur_choice, fs->master_file) == 0) {
@@ -2166,7 +2168,7 @@ alternative_evolve_slave(struct alternative *a, const char *cur_choice,
 	} else {
 		char *lnk;
 
-		xasprintf(&lnk, "%s/%s", altdir, sl->name);
+		lnk = xasprintf("%s/%s", altdir, sl->name);
 		new_file = areadlink(lnk);
 		free(lnk);
 	}
@@ -2213,11 +2215,7 @@ alternative_evolve(struct alternative *a, struct alternative *b,
 	/* Check if new slaves have been added, or existing
 	 * ones renamed. */
 	for (sl = b->slaves; sl; sl = sl->next) {
-		if (alternative_has_slave(a, sl->name))
-			alternative_evolve_slave(a, cur_choice, sl, fs);
-		else
-			sl->updated = true;
-
+		alternative_evolve_slave(a, cur_choice, sl, fs);
 		alternative_copy_slave(a, sl);
 	}
 }
@@ -2574,6 +2572,8 @@ main(int argc, char **argv)
 	char *path = NULL;
 	const char *current_choice = NULL;
 	const char *new_choice = NULL;
+	bool modifies_alt = false;
+	bool modifies_sys = false;
 	int i = 0;
 
 	setlocale(LC_ALL, "");
@@ -2725,6 +2725,21 @@ main(int argc, char **argv)
 		           "--config, --set, --set-selections, --install, "
 		           "--remove, --all, --remove-all or --auto"));
 
+	/* The following actions might modify the current alternative. */
+	if (strcmp(action, "set") == 0 ||
+	    strcmp(action, "auto") == 0 ||
+	    strcmp(action, "config") == 0 ||
+	    strcmp(action, "remove") == 0 ||
+	    strcmp(action, "remove-all") == 0 ||
+	    strcmp(action, "install") == 0)
+		modifies_alt = true;
+
+	/* The following actions might modify the system somehow. */
+	if (modifies_alt ||
+	    strcmp(action, "all") == 0 ||
+	    strcmp(action, "set-selections") == 0)
+		modifies_sys = true;
+
 	if (strcmp(action, "install") == 0)
 		alternative_check_install_args(inst_alt, fileset);
 
@@ -2751,35 +2766,28 @@ main(int argc, char **argv)
 		alternative_load(a, ALTDB_WARN_PARSER);
 	}
 
-	/* Handle actions. */
-	if (strcmp(action, "all") == 0) {
+	if (modifies_sys)
 		log_msg("run with %s", get_argv_string(argc, argv));
-		alternative_config_all();
-		exit(0);
-	} else if (strcmp(action, "get-selections") == 0) {
-		alternative_get_selections();
-		exit(0);
-	} else if (strcmp(action, "set-selections") == 0) {
-		log_msg("run with %s", get_argv_string(argc, argv));
-		alternative_set_selections(stdin, _("<standard input>"));
-		exit(0);
-	} else if (strcmp(action, "display") == 0) {
-		alternative_display_user(a);
-		exit(0);
-	} else if (strcmp(action, "query") == 0) {
-		alternative_display_query(a);
-		exit(0);
-	} else if (strcmp(action, "list") == 0) {
-		alternative_display_list(a);
-		exit(0);
+
+	if (modifies_alt) {
+		current_choice = alternative_get_current(a);
+		alternative_select_mode(a, current_choice);
 	}
 
-	/* Actions below might modify the system. */
-	log_msg("run with %s", get_argv_string(argc, argv));
-	current_choice = alternative_get_current(a);
-	alternative_select_mode(a, current_choice);
-
-	if (strcmp(action, "set") == 0) {
+	/* Handle actions. */
+	if (strcmp(action, "all") == 0) {
+		alternative_config_all();
+	} else if (strcmp(action, "get-selections") == 0) {
+		alternative_get_selections();
+	} else if (strcmp(action, "set-selections") == 0) {
+		alternative_set_selections(stdin, _("<standard input>"));
+	} else if (strcmp(action, "display") == 0) {
+		alternative_display_user(a);
+	} else if (strcmp(action, "query") == 0) {
+		alternative_display_query(a);
+	} else if (strcmp(action, "list") == 0) {
+		alternative_display_list(a);
+	} else if (strcmp(action, "set") == 0) {
 		new_choice = alternative_set_manual(a, path);
 	} else if (strcmp(action, "auto") == 0) {
 		new_choice = alternative_set_auto(a);
@@ -2810,7 +2818,8 @@ main(int argc, char **argv)
 		}
 	}
 
-	alternative_update(a, current_choice, new_choice);
+	if (modifies_alt)
+		alternative_update(a, current_choice, new_choice);
 
 	return 0;
 }
